/*
 * HttpAuth
 *
 * Copyright (C) 2004 Stefan Walter
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "usuals.h"
#include "httpauthd.h"
#include "basic.h"
#include "stringx.h"

int basic_parse(const char* header, ha_buffer_t* buf, basic_header_t* rec)
{
    char* t;
    ASSERT(header && buf && rec);
  
    memset(rec, 0, sizeof(*rec));
        
    /* 
     * Authorization header is in this format:
     * 
     * "Basic " B64(user ":" password)
     */

    header = trim_start(header);
    header = (const char*)ha_bufdec64(buf, header, NULL);
      
    if(!header)
        return CHECK_BUF(buf) ? HA_CRITERROR : HA_FALSE;
      
      
    /* We have a cache key at this point so hash it */  
    md5_string(rec->key, header);

            
    /* Parse the user. We need it in any case */
    t = strchr(header, ':');
    if(t != NULL)
    {
        /* Break the string in half */
        *t = 0;
    
        rec->user = header;
        rec->password = t + 1;
    }
  
    return HA_OK;
}
 
